import numpy as np
import matplotlib
import matplotlib.pyplot as plt

# codes for ggplot like backgroud
# http://messymind.net/making-matplotlib-look-like-ggplot/
# Or simply import seaborn

favorite_colors=["deepskyblue", "limegreen", "orangered", "cyan", "magenta", 
                 "gold", "blueviolet", "dodgerblue", "greenyellow", "tomato",
                 "turquoise", "orchid", "darkorange", "mediumslateblue"]

WeiZhu_colors = ['#4796d7', '#f79e54', '#79a702', '#df5858', '#556cab', 
                 '#de7a1f', '#ffda5c', '#4b595c', '#6ab186', '#bddbcf', 
                 '#daad58', '#488a99', '#f79b78', '#ffba00']

#seaborn_colors = seaborn.color_palette("hls", 8)

def set_colors(color_list=WeiZhu_colors):
    """
    Set the favorite colors in matplotlib color_cycle and return the 
    list of favorite colors.
    """
    matplotlib.rcParams['axes.color_cycle'] = color_list
    return color_list


def set_frame(ax):
    """Example of setting the frame of the plot box.
    """
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.spines['bottom'].set_visible(True)
    ax.spines['left'].set_visible(True)
    ax.xaxis.set_ticks_position('bottom')
    ax.yaxis.set_ticks_position('left')
    ax.get_xaxis().set_tick_params(direction='out')
    ax.get_yaxis().set_tick_params(direction='out')
    return ax


def set_style(label_size=12, grid_alpha=0.4):
    """
    Set the figure style
    """
    if grid_alpha is not None and grid_alpha >= 0 and grid_alpha <= 1:
        matplotlib.rcParams['grid.alpha'] = grid_alpha
    matplotlib.rcParams['xtick.labelsize'] = label_size
    matplotlib.rcParams['ytick.labelsize'] = label_size
    matplotlib.rcParams['legend.fontsize'] = label_size * 1.1
    matplotlib.rcParams['axes.labelsize'] = label_size * 1.1
    matplotlib.rcParams['axes.titlesize'] = label_size * 1.2
    matplotlib.rcParams['axes.titleweight'] = 'bold'


def venn3_plot(sets, set_labels=('A', 'B', 'C'), 
    set_colors=None, alpha=1.0, circle_on=False):
    """
    venn3 plot based on venn3 and venn3_circles from matplotlib_venn.

    Example:
    --------
    set1 = set(['A', 'B', 'C', 'D'])
    set2 = set(['B', 'C', 'D', 'E'])
    set3 = set(['C', 'D',' E', 'F', 'G'])
    venn3_plot([set1, set2, set3], ('Set1', 'Set2', 'Set3'))
    """
    from matplotlib_venn import venn3, venn3_circles

    if circle_on:
        v = venn3_circles(subsets=(1,1,1,1,1,1,1), alpha=0.8, color="r")
    if set_colors is None: 
        set_colors = favorite_colors[:3]
    v = venn3(subsets=(1,1,1,1,1,1,1), set_labels=set_labels, 
        set_colors=set_colors, alpha=alpha)
    v.get_label_by_id('111').set_text(len(sets[0]&sets[1]&sets[2]))
    v.get_label_by_id('110').set_text(len(sets[0]&sets[1]-sets[2]))
    v.get_label_by_id('101').set_text(len(sets[0]-sets[1]&sets[2]))
    v.get_label_by_id('100').set_text(len(sets[0]-sets[1]-sets[2]))
    v.get_label_by_id('011').set_text(len(sets[2]&sets[1]-sets[0]))
    v.get_label_by_id('010').set_text(len(sets[1]-sets[2]-sets[0]))
    v.get_label_by_id('001').set_text(len(sets[2]-sets[1]-sets[0]))

    return v


def contour2D(x, y, f, N=10, cmap="bwr", contourLine=True, optima=True, **kwargs):
    """
    Plot 2D contour.
    
    Parameters
    ----------
    x: array like (m1, )
        The first dimention
    y: array like (m2, )
        The Second dimention
    f: a function
        The funciton return f(x1, y1)
    N: int
        The number of contours
    contourLine: bool
        Turn the contour line
    optima: bool
        Turn on the optima
    **kwargs: further arguments for matplotlib.boxplot
        
    Returns
    -------
    result: contourf
        The same as the return of matplotlib.plot.contourf
        See: .. plot:: http://matplotlib.org/examples/pylab_examples/contourf_demo.html

    Example
    -------
    def f(x, y):
        return -x**2-y**2
    x = np.linspace(-0.5, 0.5, 100)
    y = np.linspace(-0.5, 0.5, 100)
    contour2D(x, y, f)
    """
    X,Y = np.meshgrid(x,y)
    Z = np.zeros(X.shape)
    for i in range(Z.shape[0]):
        for j in range(Z.shape[1]):
            Z[i,j] = f(X[i,j],Y[i,j])
    idx = np.argmax(Z)

    cf = plt.contourf(X, Y, Z, N, cmap=cmap, **kwargs)
    if contourLine is True:
        C = plt.contour(X, Y, Z, N, alpha=0.7, colors="k", linewidth=0.5)
        plt.clabel(C, inline=1, fontsize=10)
    if optima is True:
        plt.scatter(X[idx/100, idx%100], Y[idx/100, idx%100], s=120, marker='*')
                    #, marker=(5,2))
    
    plt.xlim(np.min(x), np.max(x))
    plt.ylim(np.min(y), np.max(y))
    return cf





# A general cross-validation

import numpy as np

class CrossValidation:
    def __init__(self, X, Y):
        """General cross-validation for both classification and regression.
        1) check the shape of X and Y are compatible with the input model;
        2) the model doesn't have memory on previous trainings, i.e., all 
        parameters are only based on current fit.
        """
        self.X, self.Y = X, Y
        
    def cv_regression(self, model, folds=3, shuffle=True):
        """run cross-validation for regression.
        For regression, make sure the input model has the following 
        functions: fit and predict
        """
        self.Y_pre = np.zeros(self.Y.shape[0])
        fold_len = int(self.Y.shape[0] / folds)
        idx_all = np.arange(self.Y.shape[0])
        if shuffle:
            np.random.shuffle(idx_all)
            
        for i in range(folds):
            if i < folds - 1:
                _idx = idx_all[i*fold_len : (i+1)*fold_len]
            else:
                _idx = idx_all[i*fold_len : self.Y.shape[0]]
            Xtest = self.X[_idx, :]
            Xtrain = np.delete(self.X, _idx, 0)
            Ytrain = np.delete(self.Y, _idx)
            
            model.fit(Xtrain, Ytrain)
            self.Y_pre[_idx] = model.predict(Xtest)
        return self.Y_pre
    
    def cv_classification(self, model, folds=3, shuffle=True):
        """Run cross-validation for classification.
        For classification, make sure the input model has the following 
        functions: fit, predict and predict_proba.

        For using leave-one-out, set folds=-1 or the same length of Y.
        """
        if folds == -1:
            folds = self.Y.shape[0]

        cc = np.unique(self.Y)
        self.Ystate = np.zeros(self.Y.shape[0])
        self.Yscore = np.zeros((self.Y.shape[0], len(cc)))
        idx_all = []
        fd_lens = []
        for i in range(len(cc)):
            _idx = np.where(self.Y == cc[i])[0]
            if shuffle: 
                np.random.shuffle(_idx)
            idx_all.append(_idx)
            fd_lens.append(int(len(_idx)/folds))
        
        for i in range(folds):
            idx_use = np.array([], "int")
            if (folds == -1 or folds == self.Y.shape[0]):
                idx_use = [i]
            else:
                for j in range(len(cc)):
                    if i < folds-1:
                        _idx = idx_all[j][i*fd_lens[j]: (i+1)*fd_lens[j]]
                    else:
                        _idx = idx_all[j][i*fd_lens[j]:]
                    idx_use = np.append(idx_use, _idx)
                    
            Xtest = self.X[idx_use, :]
            Xtrain = np.delete(self.X, idx_use, 0)
            Ytrain = np.delete(self.Y, idx_use)
            
            model.fit(Xtrain, Ytrain)
            self.Ystate[idx_use] = model.predict(Xtest)
            model.fit(Xtrain, Ytrain)
            self.Yscore[idx_use,:] = model.predict_proba(Xtest)
        return self.Ystate, self.Yscore

            


# This class is a mixture of linear regression.

import numpy as np

class MixLinearRegression:
    def __init__(self, prior=None, coef_=None, intercept_=None, sigma=None):
        """an initialization of the model, which could be extended
        with initial settings."""
        self.prior = prior
        self.sigma = sigma
        self.coef_ = coef_
        self.intercept_ = intercept_

    def fit(self, X, Y, K=1, run_min=100, run_max=100000, gap_ratio=0.0001, 
        is_stochastic=False):
        """implement the training"""
        # check the shape of the input
        if len(X.shape) == 1:
            X = X.reshape(-1,1)
        if len(Y.shape) == 2 and Y.shape[1] == 1:
            Y = Y.reshape(-1)
        self.X = X
        self.Y = Y
        X1 = np.ones((X.shape[0], X.shape[1]+1))
        X1[:,1:] = X
        N, M = X1.shape
        R = np.random.dirichlet(np.ones(K), N)
        Prior = np.ones(K)
        Coef  = np.ones((M, K))
        Sigma = np.ones(K)
        for i in range(run_max):
            #M-step
            Prior = R.mean(axis=0)
            if is_stochastic:
                for n in range(R.shape[0]):
                    R[n,:] = np.random.multinomial(1, R[n,:], size=1)
            for j in range(K):
                Coef[:,j] = np.linalg.inv(np.dot((X1*R[:,j:j+1]).T, X1)).dot((X1*R[:,j:j+1]).T).dot(Y)
                Sigma[j]  = np.sqrt(np.sum((Y - X1.dot(Coef[:,j]))**2 * R[:,j]) / np.sum(R[:,j]))
            #E-step
            for j in range(K):
                R[:,j] = Prior[j] / (Sigma[j] * np.sqrt(2*np.pi)) * np.exp(-0.5*(Y - X1.dot(Coef[:,j]))**2 / Sigma[j]**2)
            R[:,:] = R / np.sum(R, axis=1).reshape(-1,1)
            # check convergence
            loglik = 0
            for j in range(K):
                loglik += Prior[j] / (Sigma[j] * np.sqrt(2*np.pi)) * np.exp(-0.5*(Y - X1.dot(Coef[:,j]))**2 / Sigma[j]**2)
            loglik = np.sum(np.log(loglik))

            if i >= run_min and (loglik - loglik_old) < gap_ratio*abs(loglik):
                break
            else:
                loglik_old = loglik

        self.coef_ = Coef[1:,:]
        self.intercept_ = Coef[0,:]
        self.loglik = loglik
        self.sigma  = Sigma
        self.prior  = Prior
        self.probs  = R
        self.bic    = -2*loglik + (K*(M+2)-1)*np.log(N)
        
    def predict(self, Xtest):
        """implement the prediction"""
        if len(Xtest.shape) == 1:
            Xtest = Xtest.reshape(-1,1)
        RV = np.zeros(Xtest.shape[0])
        for j in range(self.coef_.shape[1]):
            RV += self.prior[j] * (Xtest.dot(self.coef_[:,j]) + 
                self.intercept_[j])
        return RV.reshape(-1)

    def predict_label(self, Xtest, Ytest):
        K = self.prior.shape[0]
        P = np.zeros(K, N)
        for j in range(K):
            P[:,j] = self.prior[j] * 1.0 / (self.sigma[j] * np.sqrt(2*np.pi))
            P[:,j] *= np.exp(-0.5*(Xtest.dot(self.coef_[:,j]) + 
                self.intercept_[j] - Ytest)**2 / self.sigma[j]**2)
        P[:,:] = P / np.sum(R, axis=1).reshape(-1,1)
        return P

        


# Copyright(c) 2019, The HiLearn developer (Yuanhua Huang)
# Licensed under the Apache-2.0 License

from .box_plot import *
from .dot_plot import *
from .base_plot import *
from .curve_plot import *
from .seaborn_plot import *


import numpy as np
import scipy.stats as st
import matplotlib.pyplot as plt
from sklearn import linear_model

def corr_plot(x, y, max_num=10000, outlier=0.01, line_on=True,
              corr_on=True, size=30, dot_color=None, outlier_color="r",
              alpha=0.8, color_rate=10): #"deepskyblue"
    score = st.pearsonr(x, y)
    np.random.seed(0)
    if len(x) > max_num:
        idx = np.random.permutation(len(x))[:max_num]
        x, y = x[idx], y[idx]
    outlier = int(len(x) * outlier)
    
    xy = np.vstack([x,y])
    z = st.gaussian_kde(xy)(xy)
    idx = z.argsort()
    idx1, idx2 = idx[outlier:], idx[:outlier]
    
    if dot_color is None: 
        #c_score = np.log2(z[idx]+100)
        c_score = np.log2(z[idx] + color_rate*np.min(z[idx]))
    else:
        #idx2 = []
        c_score = dot_color
    
    plt.set_cmap("Blues")
    plt.scatter(x[idx], y[idx], c=c_score, edgecolor='', s=size, alpha=alpha)
    plt.scatter(x[idx2], y[idx2], c=outlier_color, edgecolor='', s=size/5, 
                alpha=alpha/3.0)#/5
    plt.grid(alpha=0.4)

    if line_on:
        clf = linear_model.LinearRegression()
        clf.fit(x.reshape(-1,1), y)
        xx = np.linspace(x.min(), x.max(), 1000).reshape(-1,1)
        yy = clf.predict(xx)
        plt.plot(xx, yy, "k--", label="R=%.3f" %score[0])
        # plt.plot(xx, yy, "k--")

    if corr_on:
        plt.legend(loc="best", fancybox=True, ncol=1)
        # plt.annotate("R=%.3f\np=%.1e" %score, fontsize='x-large', 
        #             xy=(0.97, 0.05), xycoords='axes fraction',
        #             textcoords='offset points', ha='right', va='bottom')


def volcano_plot(fold_change, pval, p_min=0.00001, 
                 x_log10=True, p_threshold=0.05, 
                 h_color="red", label=None):
    """
    Volcano plot between log_fold change and p values, which is often used 
    for hypothesis test between two conditions.
    """
    pval[pval < p_min] = p_min
    idx1 = pval < p_threshold
    idx0 = pval >= p_threshold
    plt.scatter(fold_change[idx0], -np.log10(pval)[idx0], 
                color="grey", alpha=0.7, label=None)
    plt.scatter(fold_change[idx1], -np.log10(pval)[idx1], 
                color=h_color, alpha=0.7, label=label)
    plt.ylabel("-log10(p value)")
    plt.xlabel("Fold change")
    if x_log10: 
        plt.xscale('log', basex=10)


# some wrapped functions from seaborn
import numpy as np
import scipy.stats as st

def regplot(x, y, hue=None, hue_values=None, show_corr=True, **kwargs):
    """Wrap plot of `seaborn.regplot` with supporting hue and showing 
    correlation coeffecient.

    Parameters
    ----------
    x: `array_like`, (1, ) for values on x-axis
    y: `array_like`, (1, ) for values on y-axis
    hue: `array_like`, (1, )
        Values to stratify samples into different groups
    hue_values: list or `array_like`
        A list of unique hue values; orders are kept in layers
    show_corr: bool
        Whether show Pearson's correlation coefficient in legend
    **kwargs: for `seaborn.regplot`
    """
    import seaborn
    if hue is None:
        if show_corr:
            _label = "R=%.2f" %(st.pearsonr(x, y)[0])
        else:
            _label = None
        seaborn.regplot(x, y, label=_label, **kwargs)
    else:
        if hue_values is None:
            hue_values = np.unique(hue)
        for hue_val in hue_values:
            _idx = hue == hue_val
            if show_corr:
                _label = str(hue_val) + ": R=%.2f" %(
                    st.pearsonr(x[_idx], y[_idx])[0])
            else:
                _label = None
            seaborn.regplot(x[_idx], y[_idx], label=_label, **kwargs)
    


import numpy as np

def Geweke_Z(X, first=0.1, last=0.5):
    N = X.shape[0]
    A = X[:int(first*N)]
    B = X[int(last*N):]
    if np.sqrt(np.var(A) + np.var(B)) == 0:
        Z = None
    else:
        Z = abs(A.mean() - B.mean()) / np.sqrt(np.var(A) + np.var(B))
    return Z

def mcmc_sampler(data, prior_pdf, likelihood, propose_pdf, 
                 propose_func, xx, param, adaptive_func=None, 
                 min_run=1000, max_rum=3000, gap=100):
    # initialization
    X_now = propose_func(xx, param)
    L_now = likelihood(X_now, data, param)
    P_now = prior_pdf(X_now) + L_now
    
    # MCMC running
    accept_num = 0
    L_all = np.zeros(max_rum)
    X_all = np.zeros((max_rum, len(X_now)))
    for m in range(max_rum):
        P_try, L_try = 0.0, 0.0
        Q_now, Q_try = 0.0, 0.0
        
        # step 1: propose a value
        X_try = propose_func(X_now, param)
        Q_now = propose_pdf(X_now, X_try, param)
        Q_try = propose_pdf(X_try, X_now, param)
        L_try = likelihood(X_try, data, param)
        P_try = prior_pdf(X_try) + L_try

        # step 2: accept or reject the proposal
        alpha = np.exp(min(P_try+Q_now-P_now-Q_try, 0))
        if alpha is None: 
            print("Warning: accept ratio alpha is none!")
        elif np.random.rand(1) < alpha:
            accept_num += 1
            X_now = X_try + 0.0
            P_now = P_try + 0.0
            L_now = L_try + 0.0
        L_all[m] = L_now
        X_all[m,:] = X_now
        
        # step 3. convergence diagnostics
        if m >= min_run and m % gap == 0:
            z_scores = np.zeros(X_all.shape[1])
            for k in range(X_all.shape[1]):
                z_scores[k] = Geweke_Z(X_all[:m, k])
            if sum(z_scores <= 2) == len(z_scores):
                L_all = L_all[:m]
                X_all = X_all[:m,:]
                break

        # step 4: adaptive MCMC
        if (adaptive_func is not None and 
            accept_num >= 10 and m % gap == 0):
            param = adaptive_func(X_all[:m,:], param)

    print("MCMC summary: %d acceptance in %d run (%.1f%%)." 
          %(accept_num, m, accept_num*100.0/m))
    return X_all, L_all, accept_num



import numpy as np

def logistic(x):
    """
    Logistic function, mapping (-inf, inf) to (0,1)

    Parameters
    ----------
    x: float, int, array, list
        input variable

    Returns
    -------
    val: float, int, array
        logistic(x)
    """
    return np.exp(x)/(1+np.exp(x))
    
def logit(x, minval=0.001):
    """
    Logit function, mapping (0,1) to (-inf, inf)

    Parameters
    ----------
    x: float, int, array, list
        input variable
    minval: float (optional, default=0.001)
        minimum value of input x, and maximum of 1-x

    Returns
    -------
    val: float, int, array
        logit(x)
    """
    if isinstance(x,  (list, tuple, np.ndarray)):
        x[1-x<minval] = 1-minval
        x[x<minval] = minval
    else:
        x = max(minval, x)
        x = min(1-minval, x)
    val = np.log(x/(1-x))
    return val

def uninormal_pdf(x, mu, sigma, log=False):
    """
    Probability density function of univariate normal distribution

    Parameters
    ----------
    x: float, int, array, list
        input variable
    mu: float
        mean of the normal distribution
    sigma: float
        standard divation of the normal distibution
    log: bool (optional)
        If True, return pdf at log scale

    Returns
    -------
    pdf: float, int, array
        Probability density of x
    """
    mu = np.float(mu)
    sigma = np.float(sigma)
    pdf = np.log(1/(sigma*np.sqrt(2*np.pi))) - 0.5*((x-mu)/sigma)**2
    if log is False:
        pdf = np.exp(pdf)
    return pdf


def permutation_test(x1, x2, times=1000, sides=2, metrics="mean", seed=None):
    """
    Permutation test: whether group x1 has equal [mean|median] as group x2

    Parameters
    ----------
    x1: array or list of int/float
        samples for variable x1
    x2: array or list of int/float
        samples for variable x2
    times: int
        number of iterations, better <= 10^5
    sides: int, 1 or 2
        one-sided p value or two-sided p-value
    metrics: str
        the metrics: mean, median
    seed: int or None
        the seed for permutation

    Returns
    -------
    (deta, pval): float, float
        the difference between two groups and the p value

    Examples
    --------
    >>>x1 = np.random.normal(0,1,100)
    >>>x2 = np.random.normal(1,2,100)
    >>>permutation_test(x1, x2)
    """

    x1 = np.array(x1)
    x2 = np.array(x2)
    x1 = x1[x1 == x1]
    x2 = x2[x2 == x2]

    if metrics == "median":
        diff_obs = np.median(x1) - np.median(x2)
    else:
        diff_obs = np.mean(x1) - np.mean(x2)

    if seed is not None:
        np.random.seed(seed)

    cnt = 0
    xx = np.append(x1, x2)
    diff_all = np.zeros(times)
    for i in range(times):
        # np.random.shuffle(xx)
        xx = np.random.permutation(xx)
        if metrics == "median":
            diff_all[i] = np.median(xx[:len(x1)]) - np.median(xx[len(x1):])
        else:
            diff_all[i]  = np.mean(xx[:len(x1)]) - np.mean(xx[len(x1):])

    if sides == 2:
        pval = np.mean(np.abs(diff_all) >= np.abs(diff_obs))
    else:
        pval = np.mean(diff_all >= np.abs(diff_obs))

    return diff_obs, pval



# Copyright(c) 2019, The HiLearn developer (Yuanhua Huang)
from .version import __version__
from .base import match, id_mapping
from .stats import permutation_test
from .cross_validation import CrossValidation
from .mixture_model.mix_linear_regression import MixLinearRegression

from .plot import *





import numpy as np
import matplotlib.pyplot as plt

def boxgroup(x, labels=None, conditions=None, colors=None, notch=False, sys='',
             widths=0.9, patch_artist=True, showmeans=True, alpha=1, **kwargs):
    """
    Make boxes for a multiple groups data in different conditions.
    
    Parameters
    ----------
    x: a list of multiple groups
        The input data, e.g., [group1, group2, ..., groupN]. 
        If there is only one group, use [group1]. For each gorup,
        it can be an array or a list, containing the same number of conditions.
    labels: a list or an array
        The names of each group memeber
    conditions: a list or an array
        The names of each conditions
    **kwargs: further arguments for matplotlib.boxplot
        
    Returns
    -------
    result: dict
        The same as the return of matplotlib.boxplot
        See: .. plot:: mpl_examples/statistics/boxplot_demo.py

    Example
    -------
    data1 = [np.random.rand(50), np.random.rand(30)-.2, np.random.rand(10)+.3]
    data2 = [np.random.rand(40), np.random.rand(10)-.2, np.random.rand(15)+.3]

    plt.subplot(1,2,1)
    boxgroup([data1, data2], labels=("G1","G2","G3"), conditions=["C1","C2"])
    plt.subplot(1,2,2)
    boxgroup(data1, conditions=("G1","G2","G3"))
    """
    box_data = []
    cond_num = len(x)
    cond_loc = np.zeros(len(x))
    x_loc = np.array([])
    for i in range(len(x)):
        if type(x[i]) == np.ndarray:
            if len(x[i].shape) == 1:
                temp_loc = np.arange(1)
                box_data.append(x[i])
                group_num = 1
            else:
                group_num = x[i].shape[1]
                temp_loc = np.arange(x[i].shape[1])
                for j in range(x[i].shape[1]):
                    box_data.append(x[i][:,j])
        else:
            box_data += x[i]
            group_num = len(x[i])
            temp_loc = np.arange(len(x[i]))
        if i == 0:
            x_loc = temp_loc
            cond_loc[i] = np.mean(temp_loc)
        else:
            cond_loc[i] = np.mean(temp_loc)+x_loc[-1]+2
            x_loc = np.append(x_loc, x_loc[-1]+2+temp_loc)
        
    bp = plt.boxplot(box_data, notch, sys, positions=x_loc, widths=widths, 
                    patch_artist=patch_artist, **kwargs)
    if colors is None:
        colors = favorite_colors
    
    for i in range(len(box_data)):
        bp['medians'][i].set(color='blue', linewidth=2, alpha=alpha)
        bp['caps'][i].set(color='grey', linewidth=0, alpha=alpha)
        bp['caps'][i+len(box_data)].set(color='grey', linewidth=0, 
            alpha=alpha)
        bp['whiskers'][i].set(linestyle='solid', color='grey', 
            linewidth=2, alpha=alpha)
        bp['whiskers'][i+len(box_data)].set(linestyle='solid', 
            color='grey', linewidth=2, alpha=alpha)
        bp['boxes'][i].set(color=colors[i%group_num], 
            linewidth=2, alpha=alpha)
        
        if showmeans is True:
            plt.plot([x_loc[i]], [np.mean(box_data[i])],
                     color='w', marker='*', markersize=9)#, markeredgecolor='k'
        
    if labels is not None:
        for i in range(group_num):
            plt.scatter([], [], s=150, c=colors[i], marker='s', 
                        edgecolor='none', alpha=alpha, label=labels[i])
            
    if conditions is not None:
        plt.xticks(cond_loc, conditions)
    
    plt.xlim(x_loc[0]-0.7, x_loc[-1]+0.7)
    plt.legend(loc="best", scatterpoints=1, fancybox=True, ncol=group_num)
    plt.grid(alpha=0.4)

    return bp



import numpy as np
import matplotlib.pyplot as plt

def ROC_plot(state, scores, threshold=None, color=None, legend_on=True, 
             label="predict", base_line=True, linewidth=1.0):
    """
    Plot ROC curve and calculate the Area under the curve (AUC) from the
    with the prediction scores and true labels.
    The threshold is the step of the ROC cureve.
    """
    # if color is None or color=="none": 
    #     color = np.random.rand(3,1)
    
    score_gap = np.unique(scores)
    if len(score_gap) > 2000:
        idx = np.random.permutation(len(score_gap))
        score_gap = score_gap[idx[:2000]]
    score_gap = np.append(np.min(score_gap)-0.1, score_gap)
    score_gap = np.append(score_gap, np.max(score_gap)+0.1)
    if threshold is not None:
        thresholds = np.sort(np.append(threshold, score_gap))
        _idx = np.where(scores >= threshold)[0]
        _fpr = np.sum(state[_idx] == 0)/np.sum(state == 0).astype('float')
        _tpr = np.sum(state[_idx] == 1)/np.sum(state == 1).astype('float')
        # plt.scatter(_fpr, _tpr, marker="o", s=80, facecolors='none', edgecolors=color)
        if color is None:
            plt.plot(_fpr, _tpr, marker='o', markersize=8, mfc='none')
        else:
            plt.plot(_fpr, _tpr, marker='o', markersize=8, mec=color, mfc=color) 
    else:
        thresholds = np.sort(score_gap)
    #thresholds = np.arange(np.min(threshold), 1+2*threshold, threshold)
    
    fpr, tpr = np.zeros(thresholds.shape[0]), np.zeros(thresholds.shape[0])
    for i in range(thresholds.shape[0]):
        idx = np.where(scores >= thresholds[i])[0]
        fpr[i] = np.sum(state[idx] == 0)/np.sum(state == 0).astype('float')
        tpr[i] = np.sum(state[idx] == 1)/np.sum(state == 1).astype('float')
        
    auc = 0
    for i in range(thresholds.shape[0]-1):
        auc = auc + (fpr[i]-fpr[i+1]) * (tpr[i]+tpr[i+1]) / 2.0
        
    if color is None:
        plt.plot(fpr, tpr, "-",  linewidth=linewidth,
                 label="%s: AUC=%.3f" %(label,auc))
    else:
        plt.plot(fpr, tpr, "-",  linewidth=linewidth, color=color,
                 label="%s: AUC=%.3f" %(label,auc))
    if base_line: plt.plot(np.arange(0,2), np.arange(0,2), "k--", linewidth=1.0,
        label="random: AUC=0.500")
        
    if legend_on:
        plt.legend(loc="best", fancybox=True, ncol=1)
    
    plt.xlabel("False Positive Rate (1-Specificity)")
    plt.ylabel("True Positive Rate (Sensitivity)")
    return fpr, tpr, thresholds, auc


def PR_curve(state, scores, threshold=None, color=None, legend_on=True,  
             label="predict", base_line=False, linewidth=1.5):
    """
    Plot ROC curve and calculate the Area under the curve (AUC) from the
    with the prediction scores and true labels.
    The threshold is the step of the ROC cureve.
    """

    ###Test compare
    # from sklearn.metrics import precision_recall_curve,average_precision_score
    # precision, recall, thresholds = precision_recall_curve(labels, BF_tmp)
    # ap = average_precision_score(labels, BF_tmp)
    # plt.plot(recall, precision, label="%.3f" %(ap))

    if color is None or color=="none": 
        color = np.random.rand(3,1)
    
    score_gap = np.unique(scores)
    if len(score_gap) > 2000:
        idx = np.random.permutation(len(score_gap))
        score_gap = score_gap[idx[:2000]]
    #score_gap = np.append(np.min(score_gap)-0.1, score_gap)
    #score_gap = np.append(score_gap, np.max(score_gap)+0.1)
    if threshold is not None:
        thresholds = np.sort(np.append(threshold, score_gap))
        idx1 = np.where(scores >= threshold)[0]
        idx2 = np.where(scores <  threshold)[0]
        FP = np.sum(state[idx1] == 0)
        TP = np.sum(state[idx1] == 1)
        FN = np.sum(state[idx2] == 1)
        _pre = (TP+0.0)/(TP + FP)
        _rec = (TP+0.0)/(TP + FN)
        # plt.plot(_rec, _pre, marker='*', markersize=9, mec="k", mfc='none')
        plt.plot(_rec, _pre, marker='o', markersize=8, mec=color, mfc=color)
    else:
        thresholds = np.sort(score_gap)
    
    pre, rec = np.zeros(thresholds.shape[0]), np.zeros(thresholds.shape[0])
    for i in range(thresholds.shape[0]):
        idx1 = np.where(scores >= thresholds[i])[0]
        idx2 = np.where(scores <  thresholds[i])[0]
        FP = np.sum(state[idx1] == 0)
        TP = np.sum(state[idx1] == 1)
        FN = np.sum(state[idx2] == 1)
        pre[i] = (TP+0.0)/(TP + FP)
        rec[i] = (TP+0.0)/(TP + FN)
        
    auc = 0
    _rec = np.append(1.0, rec)
    _pre = np.append(0.0, pre)
    _rec = np.append(_rec, 0.0)
    _pre = np.append(_pre, 1.0)
    for i in range(_rec.shape[0]-1):
        auc = auc + (_rec[i]-_rec[i+1]) * (_pre[i]+_pre[i+1]) / 2.0
        
    plt.plot(_rec, _pre, "-", color=color, linewidth=linewidth, 
             label="%s: AUC=%.3f" %(label,auc))
    if base_line: plt.plot(np.arange(0,2), 1-np.arange(0,2), "k--", 
                           linewidth=1.0, label="random: AUC=0.500")
        
    if legend_on:
        plt.legend(loc="best", fancybox=True, ncol=1)
    
    plt.ylabel("Precision: TP/(TP+FP)")
    plt.xlabel("Recall: TP/(TP+FN)")
    return rec, pre, thresholds, auc



def ecdf_plot(data, x=None, **kwargs):
    """
    Empirical plot for cumulative distribution function
    
    Parameters
    ----------
    data: array or list
        data for the empirical CDF plot
    x: array or list (optional)
        the points to show the plot
    **kwargs: 
        **kwargs for matplotlib.plot
        
    Returns
    -------
    x: array
        sorted x
    ecdf_val:
        values of empirical cdf for x
    """
    data = np.sort(np.array(data))
    if x is None:
        x = data
    else:
        x = np.sort(np.array(x))
        
    ecdf_val = np.zeros(len(x))
    for i in range(len(x)):
        ecdf_val[i] = np.mean(data < x[i])
    
    plt.plot(x, ecdf_val, **kwargs)
    return x, ecdf_val


"""
HiLearn: A small library of machine learning methods.
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Set __version__ for the project.
exec(open("./hilearn/version.py").read())

# Get the long description from the relevant file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()
    
reqs = ['numpy', 'scipy', 'scikit-learn', 'matplotlib']

setup(
    name='hilearn',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=__version__,

    description='HiLearn: A small library of machine learning methods.',
    long_description=long_description,

    # The project's main homepage.
    url='https://github.com/huangyh09/hilearn',

    # Author details
    author='Yuanhua Huang',
    author_email='yuanhua@hku.hk',

    # Choose your license
    license='Apache-2.0',

    # What does your project relate to?
    keywords=['machine learning', 'data visualisation'],

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(),#[''],#

    entry_points={
          'console_scripts': [
              ],
          }, 

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    
    install_requires=reqs,

    py_modules = ['hilearn']

    # buid the distribution: python setup.py sdist
    # upload to pypi: twine upload dist/...

)


# some basic functions

import numpy as np

def id_mapping(IDs1, IDs2, uniq_ref_only=True):
    print("Please use `match` function in future.")
    return match(IDs1, IDs2, uniq_ref_only=True)

def match(ref_ids, new_ids, uniq_ref_only=True):
    """
    Mapping new_ids to ref_ids. ref_ids can have repeated values, but new_ids 
    can only have unique ids or values. Therefore, new_ids[RT_idx] will be 
    the same as ref_ids. Note, 
    
    Parameters
    ----------
    ref_ids : array_like or list
        ids for reference with type of int, float, or string
    new_ids : array_like or list
        ids waiting to map.
        
    Returns
    -------
    RV_idx : array_like, the same length of ref_ids
        The index for new_ids mapped to ref_ids. If an id in ref_ids does not 
        exist in new_ids, then return a None for that id. 

    Examples
    --------
    >>> x1 = [5, 9, 1]
    >>> x2 = [1, 2, 5, 7, 9]
    >>> hilearn.match(x1, x2)
    array([2, 4, 0])
    >>> hilearn.match(x2, x1)
    array([2, None, 0, None, 1], dtype=object)
    >>> RT_idx = hilearn.match(x2, x1)
    >>> idx1 = np.where(RT_idx != None)[0]
    >>> idx1
    array([0, 2, 4])
    >>> idx2 = RT_idx[idx1].astype(int)
    >>> idx2
    array([2, 0, 1])
    """
    idx1 = np.argsort(ref_ids)
    idx2 = np.argsort(new_ids)
    RT_idx1, RT_idx2 = [], []
    
    i, j = 0, 0
    while i < len(idx1):
        if j == len(idx2) or ref_ids[idx1[i]] < new_ids[idx2[j]]:
            RT_idx1.append(idx1[i])
            RT_idx2.append(None)
            i += 1
        elif ref_ids[idx1[i]] == new_ids[idx2[j]]:
            RT_idx1.append(idx1[i])
            RT_idx2.append(idx2[j])
            i += 1
            if uniq_ref_only: j += 1
        elif ref_ids[idx1[i]] > new_ids[idx2[j]]:
            j += 1
            
    origin_idx = np.argsort(RT_idx1)
    RT_idx = np.array(RT_idx2)[origin_idx]
    return RT_idx
